#!/bin/bash

# Author: Victor Correal (victor.correal@bsc.es)
# Date  : 28/06/2024

UTILS_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
source ${UTILS_DIR}/../../bash_utils/helpers.sh

usage(){
    info "Usage: run_nemo.sh "
    info "\t--bin    <NEMO_BIN>"
    info "\t--env    <ENV_FILE"
    info "\t--rundir <RUNDIR_PATH>"
    info "\t--steps  <STEPS>"
    info "\t--ref-rundir <RESOLUTION>"
    info "OPTIONAL FLAGS:"
    info "[--nodes=<nodes>] [--MPI=<val>] [--OpenMP=<val>] "
    info "[--account=<SLURM-ACC>] [--queue=<queue>] [--wait] [--contiguous]"
    info "[--force] [--help] [--monitor] [--extrae] [--profile]"
}

parse_args(){
    OPTIONS=$(getopt -o "" --long bin:,env:,rundir:,steps:,ref-rundir:,nodes:,account:,queue:,MPI:,OpenMP:,timelimit,force,monitor,help,wait,contiguous,extrae,profile -n "$0" -- "$@")

    eval set -- "$OPTIONS"

    while true; do
        case "$1" in
        --bin)
            BIN_FILE="$2"
            shift 2
        ;;
        --env)
            ENV_FILE="$2"
            shift 2
        ;;
        --rundir)
            RUNDIR="$2"
            shift 2
        ;;
        --steps)
            STEPS="$2"
            shift 2
        ;;
        --ref-rundir)
            REF_RUNDIR="$2"
            shift 2
        ;;
        --nodes)
            NODES="$2"
            shift 2
        ;;
        --account)
            ACCOUNT="$2"
            shift 2
        ;;
        --queue)
            QUEUE="$2"
            shift 2
        ;;
        --MPI)
            MPI_PER_NODE="$2"
            shift 2
        ;;
        --OpenMP)
            OMP_PER_TASK="$2"
            shift 2
        ;;
        --timelimit)
            TIME_LIMIT="$2"
            shift 2
        ;;
        --help)
            usage
            exit 0
        ;;
        --monitor)
            MONITOR=true
            shift
        ;;

        --force)
            FORCE_RUN=true
            shift
        ;;
        --wait)
            SLURM_WAIT=true
            shift
        ;;
        --contiguous)
            SLURM_CONTIGUOUS=true
            shift
        ;;
        --extrae)
            USE_EXTRAE=true
            shift
        ;;
        --profile)
            PROFILE=true
            shift
        ;;
        --)
            shift
            break
        ;;
        *)
            usage
            exit 1
        ;;
        esac
    done
}


check_required_args(){
    [ -z $BIN_FILE ] && fatal "NEMO binary not set, use --bin"
    [ -z $ENV_FILE ] && fatal "ENV file not set use --env"
    [ -z $RUNDIR ] && fatal "RUNDIR not set, use --rundir"
    [ -z $REF_RUNDIR ] && fatal "REF_RUNDIR not set, user --ref-rundir"
    [ -z $STEPS ] &&  info "STEPS not set, use --steps . Using 50 as default" && STEPS=50
}

check_dirs(){
    [ ! -f $BIN_FILE ] && fatal "Unable to find bin file at $BIN_FILE"
    [ ! -f $ENV_FILE ] && fatal "Unable to find env file at $ENV_FILE"
    [ -d $RUNDIR ] && fatal "Rundir dir already exists"
    [ ! -d $REF_RUNDIR ] && fatal "Unable to find ref rundir at $REF_RUNDIR"
}

check_slurm_options(){
    if [ "$ACCOUNT" != "DEFAULT" ]; then
        SLURM_ARGS+="-A $ACCOUNT "
        info "Overriding default for SLURM ACCOUNT: $ACCOUNT"
    fi
    if [ "$QUEUE" != "DEFAULT" ]; then
        SLURM_ARGS+="--qos $QUEUE "
        info "Overiding default for QUEUE: $QUEUE"
    fi
    if [ "$NODES" != "DEFAULT" ]; then
        SLURM_ARGS+="-N $NODES "
        info "Overiding default for NODES: $NODES"
    fi
    if [ "$TIME_LIMIT" != "DEFAULT" ]; then
        SLURM_ARGS+="--time $TIME_LIMIT "
        info "Overiding default for TIME_LIMIT: $TIME_LIMIT"
    fi

    if [ "$MPI_PER_NODE" != "DEFAULT" ]; then
        SLURM_ARGS+="--ntasks-per-node $MPI_PER_NODE "
        info "Overiding default for MPI_PER_NODE: $MPI_PER_NODE"
    fi

    if [ "$OMP_PER_TASK" != "DEFAULT" ]; then
        SLURM_ARGS+="--cpus-per-task $OMP_PER_TASK "
        info "Overiding default for OMP_PER_TASK: $OMP_PER_TASK"
    fi
    if [ $SLURM_WAIT = true ]; then
        SLURM_ARGS+="-W "
        info "Waiting sbatch to finish the job before exit sbatch!"
    fi
    if [ $SLURM_CONTIGUOUS = true ]; then
        SLURM_ARGS+="--contiguous "
        info "Using SLURM contiguous"
    fi
    SLURM_ARGS+=" --exclusive -C perfparanoid"
}


print_options(){
    info "=============================MODEL============================="
    info "BIN: $BIN_FILE"
    info "ENV: $ENV_FILE"
    info "==========================JOB OPTIONS=========================="
    info "STEPS: $STEPS"
    info "RUNDIR: $RUNDIR"
    info "REF_RUNDIR: $REF_RUNDIR"
    [ $SLURM_WAIT = true ] && info "SLURM_WAIT: ON"
    [ $SLURM_CONTIGUOUS = true ] && info "SLURM_CONTIGUOS: ON"
    [ $USE_EXTRAE = true ] && info "EXTRAE ON!"
    info "====================ALLOCATION RESOURCES======================="
    info "$SLURM_ARGS"
    info "==============================================================="
}

check_user(){
    if [ $FORCE_RUN = false ]; then
        echo "FINE? (Y/N)"
        read ANSW
        case "$ANSW" in
            [Yy])
                success "RUNNING!"
                ;;
            *)
                fatal "USER INPUT $ANSW"
                ;;
        esac

    else
        echo "FORCE RUN enable"
    fi
}
